 function [TK TG] = f_strain_factors_layered_inclusion_Herve_Zaoui(n,k,g,nu,r)

%
% Calculation of the dilute concentration factor for spherical inclusion 
% with coating
% f_strain_factors_layered_inclusion_Herve_Zaoui(n,k,g,nu,r)
%
% input: number of layers within the inclusion, n [-]
%        bulk moduli k (in vectors, ordered from centre of inclusion, incl. matrix) [Pa]
%        shear moduli g (in vectors, ordered from centre of inclusion, incl. matrix) [Pa]
%        Poisson ratios nu (in vectors, ordered from centre of inclusion, incl. matrix) [-]
%        radii (ordered from centre of inclusion) [m]
%
% output: constraint constants for shperical inhomogeneity and its coating(s)
%
% written by Venca 16/04/2012

%% Hydrostatic part

N = zeros(2,2,n);
Q = zeros(2,2,n);

for i = 1:n
    N(:,:,i) = 1/(3*k(i+1)+4*g(i+1))*[3*k(i)+4*g(i+1), 4*(g(i+1)-g(i))/r(i)^3;
                                      3*(k(i+1)-k(i))*r(i)^3, 3*k(i+1)+4*g(i)];

end

Q(:,:,1) =  N(:,:,1);
for i = 2:n
    Q(:,:,i) = N(:,:,i)*Q(:,:,i-1);
end

F = zeros(n,1);
F(1) = 1/Q(1,1,n);
for i = 2:n
    F(i) = Q(1,1,i-1)/Q(1,1,n);
end
TK = F;

%% Deviatoric part

M = zeros(4,4,n);
a = zeros(1,n);
b = zeros(1,n);
c = zeros(1,n);
d = zeros(1,n);
e = zeros(1,n);
f = zeros(1,n);
alpha = zeros(1,n);

for i = 1:n
    a(i) = g(i)/g(i+1)*(7+5*nu(i))*(7-10*nu(i+1))-(7-10*nu(i))*(7+5*nu(i+1));
    b(i) = 4*(7-10*nu(i))+g(i)/g(i+1)*(7+5*nu(i));
    c(i) = 7-5*nu(i+1)+2*g(i)/g(i+1)*(4-5*nu(i+1));
    d(i) = 7+5*nu(i+1)+4*g(i)/g(i+1)*(7-10*nu(i+1));
    e(i) = 2*(4-5*nu(i))+(g(i)/g(i+1))*(7-5*nu(i));
    f(i) = (4-5*nu(i))*(7-5*nu(i+1))-(g(i)/g(i+1))*(4-5*nu(i+1))*(7-5*nu(i));
    alpha(i) = g(i)/g(i+1)-1;
end
    
for i = 1:n
    M(:,:,i) = 1/(5*(1-nu(i+1)))*...
        [c(i)/3,r(i)^2*(3*b(i)-7*c(i))/(5*(1-2*nu(i))),-12*alpha(i)/(r(i)^5),4*(f(i)-27*alpha(i))/(15*(1-2*nu(i))*r(i)^3);
         0,(1-2*nu(i+1))*b(i)/(7*(1-2*nu(i))),-20*(1-2*nu(i+1))*alpha(i)/(7*r(i)^7),-12*alpha(i)*(1-2*nu(i+1))/(7*(1-2*nu(i))*r(i)^5);
         r(i)^5*alpha(i)/2,-r(i)^7*(2*a(i)+147*alpha(i))/(70*(1-2*nu(i))),d(i)/7,r(i)^2*(105*(1-nu(i+1))+12*alpha(i)*(7-10*(nu(i+1)))-7*e(i))/(35*(1-2*nu(i)));
         -5/6*(1-2*nu(i+1))*alpha(i)*r(i)^3,7*(1-2*nu(i+1))*alpha(i)*r(i)^5/(2*(1-2*nu(i))),0,(1-2*nu(i+1))*e(i)/(3*(1-2*nu(i)))];
end

P(:,:,1) =  M(:,:,1);
for i = 2:n
    P(:,:,i) = M(:,:,i)*P(:,:,i-1);
end

A = zeros(n,1);
B = zeros(n,1);
A(1) = P(2,2,n)/(P(1,1,n)*P(2,2,n)-P(1,2,n)*P(2,1,n));
B(1) = -P(2,1,n)/(P(1,1,n)*P(2,2,n)-P(1,2,n)*P(2,1,n));

for i = 2:n
    W = 1/(P(2,2,n)*P(1,1,n)-P(1,2,n)*P(2,1,n))*P(:,:,i-1)*[P(2,2,n);-P(2,1,n);0;0];
    A(i) = W(1);
    B(i) = W(2);
end

TG = zeros(n,1);
TG(1) = A(1)-21/5*r(1)^2/(1-2*nu(1))*B(1);
for i = 2:n
    TG(i) = A(i)-21/5*(r(i)^5-r(i-1)^5)/((1-2*nu(i))*(r(i)^3-r(i-1)^3))*B(i);
end

end



                         